@@ -2,17 +2,19 @@
 Brings mathematical sets into Rust.
 */
 #[derive(Debug, Clone)]
-pub struct Set<T> {
+pub struct Set<'a, T> {
     elements:        Vec<T>,
     cardinality:     usize,
+    parent:          Option<&'a Set<'a, T>>,
 }
 
 // Main impl
-impl<T: PartialEq + Copy + Ord> Set<T> {
+impl<'a, T: PartialEq + Copy + Ord> Set<'a, T> {
 
-    pub fn new(values: Vec<T>) -> Set<T> {
+    pub fn new(values: Vec<T>) -> Set<'a, T> {
         Set { elements:    values.clone(),
               cardinality: values.len(),
+              parent:      None,
             }
     }
     /**
@@ -25,9 +27,10 @@ impl<T: PartialEq + Copy + Ord> Set<T> {
     # Returns
     A child Set.
     */
-    pub fn new_from_parent<F: Fn(T) -> bool>(parent: &Set<T>, f: F) -> Set<T> {
+    pub fn new_from_parent<F: Fn(T) -> bool>(parent: &'a Set<T>, f: F) -> Set<'a, T> {
             let mut res: Set<T> = Set { elements:    Vec::new(),
                                         cardinality: 0,
+                                        parent:      Option::Some(parent),
             };
             for elem in &parent.elements {
                 if f(*elem) {
@@ -37,7 +40,6 @@ impl<T: PartialEq + Copy + Ord> Set<T> {
             res.cardinality = res.elements.len();
             res
     }
-
     /**
     Does a mathematical union on two sets.
 
@@ -51,6 +53,7 @@ impl<T: PartialEq + Copy + Ord> Set<T> {
     pub fn union(&self, other: &Set<T>) -> Set<T> {
         let mut res: Set<T> = Set {elements:    Vec::new(),
                                    cardinality: 0,
+                                   parent:      None,
         };
 
         res.elements.append(&mut self.elements.clone());
@@ -80,10 +83,34 @@ impl<T: PartialEq + Copy + Ord> Set<T> {
         res.cardinality = res.elements.len();
         res
     }
+    /**
+    Lets you check for an element in a set.
 
-    // I know that getters and setters are VERY controversial.
-    // I'm not going to change it because the cardinality is only something
-    // You'll need to read, not set.
+    # Arguments
+    * `elem` - The element to check for.
+
+    # Returns
+    A boolean value which determines if the element is in the set.
+    */ 
+    pub fn has_element(&self, elem: &T) -> bool {
+        self.elements.contains(elem)
+    }
+}
+
+impl<T> Set<'_, T> {
+    /**
+    Lets you check wether a set has a parent or not.
+
+    # Returns
+    A boolean value which determines if the set has a value.
+    */ 
+    pub fn has_parent(&self) -> bool {
+        self.parent.is_some()
+    }
+    /* I know that getters and setters are VERY controversial.
+       I'm not going to change it because the cardinality is only something
+       You'll need to read, not set.
+    */
     pub fn cardinality(&self) -> &usize {
         &self.cardinality
     }
@@ -100,7 +127,6 @@ impl<T: PartialEq + Copy + Ord> Set<T> {
         self.elements    = vals;
         self.cardinality = self.elements.len();
     }
-
     /**
     Lets you get the elements of a set.
 
@@ -113,23 +139,41 @@ impl<T: PartialEq + Copy + Ord> Set<T> {
     pub fn elements(&self) -> &Vec<T> {
         &self.elements
     }
+}
 
+impl<T: ToString> Set<'_, T> {
     /**
-    Lets you check for an element in a set.
-
-    # Arguments
-    * `elem` - The element to check for.
+    Lets you print a set with all its parents recursively.
 
     # Returns
-    A boolean value which determines if the element is in the set.
+    Nothing.
     */ 
-    pub fn has_element(&self, elem: &T) -> bool {
-        self.elements.contains(elem)
+    pub fn full_print(&self) {
+        print!("{}", self.backend_full_print(&mut String::new()));
+    }
+    /**
+    Converts a set with all subsets to a string.
+
+    # Returns
+    A String containing the result.
+    */ 
+    pub fn to_full_string(&self) -> String {
+        self.backend_full_print(&mut String::new())
+    }
+
+    fn backend_full_print(&self, string: &mut String) -> String {
+        string.push_str(&self.to_string()); // The child-set at the bottom
+        match self.parent.is_some() {
+            true  => { string.push_str(" ⊆ "); // Add subset-character
+                       self.parent.unwrap().backend_full_print(string); } // Recursively append parent sets
+            false => { string.push_str("\n"); }
+        }
+        string.to_string()
     }
 }
 
 // Indexing for Sets
-impl<T> std::ops::Index<usize> for Set<T> {
+impl<T> std::ops::Index<usize> for Set<'_, T> {
     type Output = T;
     fn index(&self, index: usize) -> &Self::Output {
         self.elements.get(index).unwrap()
@@ -137,18 +181,18 @@ impl<T> std::ops::Index<usize> for Set<T> {
 }
 
 // Implement Printing
-impl<T: ToString> std::fmt::Display for Set<T> {
+impl<T: ToString> std::fmt::Display for Set<'_, T> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         let mut res: String = "{ ".to_owned();
         for elem in &self.elements {
             res = res + "" + &*elem.to_string() + "; ";
         }
-        write!(f, "{} }}", res)
+        write!(f, "{}}}", res)
     }
 }
 
 // Implement Equality
-impl<T: PartialEq> PartialEq for Set<T> {
+impl<T: PartialEq> PartialEq for Set<'_, T> {
     fn eq(&self, other: &Self) -> bool {
         self.elements == other.elements
     }
