@@ -36,4 +36,15 @@ fn test_map_to() {
     let result: f32 = 5f32.map_to(0., 10., 0., 1.); // Original value 5 in the range from 0-10
     std::println!("{}", result.to_string());
     println!("Should be: 0.5")
+}
+
+#[test]
+fn test_rec_printing() {
+    use crate::math::sets;
+
+    let s = sets::Set::new(vec![0,1,2,3,4,5,6,7,8,9,10]);
+    let s1 = sets::Set::new_from_parent(&s, |x| x % 2 == 0);
+    let s2 = sets::Set::new_from_parent(&s1, |x| x == 4);
+
+    s2.full_print();
 }
\ No newline at end of file
